/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyInteger;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.item.ItemStack;
import net.minecraft.util.DamageSource;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.objects.blocks.property.ILightableBlock;

@ParametersAreNonnullByDefault
public class BlockMolten extends Block implements ILightableBlock
{
    public static final PropertyInteger LAYERS = PropertyInteger.func_177719_a("layers", 1, 4);

    private static final AxisAlignedBB[] MOLTEN_AABB = new AxisAlignedBB[] {
        new AxisAlignedBB(0, 0, 0, 1, 0.25, 1),
        new AxisAlignedBB(0, 0, 0, 1, 0.5, 1),
        new AxisAlignedBB(0, 0, 0, 1, 0.75, 1),
        field_185505_j
    };

    public BlockMolten()
    {
        super(Material.field_151576_e);
        func_149711_c(-1);
        func_180632_j(this.func_176194_O().func_177621_b().func_177226_a(LIT, false).func_177226_a(LAYERS, 1));
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_185481_k(IBlockState state)
    {
        return false;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149730_j(IBlockState state)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    @Nonnull
    public IBlockState func_176203_a(int meta)
    {
        return func_176223_P().func_177226_a(LAYERS, (meta & 0b11) + 1).func_177226_a(LIT, meta > 3);
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return state.func_177229_b(LAYERS) + (state.func_177229_b(LIT) ? 4 : 0) - 1;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149637_q(IBlockState state)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149721_r(IBlockState state)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149686_d(IBlockState state)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    @Nonnull
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        return MOLTEN_AABB[state.func_177229_b(LAYERS) - 1];
    }

    @SuppressWarnings("deprecation")
    @Nullable
    @Override
    public AxisAlignedBB func_180646_a(IBlockState blockState, IBlockAccess worldIn, BlockPos pos)
    {
        return MOLTEN_AABB[blockState.func_177229_b(LAYERS) - 1];
    }

    @SuppressWarnings("deprecation")
    @SideOnly(Side.CLIENT)
    @Override
    @Nonnull
    public AxisAlignedBB func_180640_a(IBlockState state, World worldIn, BlockPos pos)
    {
        return MOLTEN_AABB[state.func_177229_b(LAYERS) - 1];
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149662_c(IBlockState state)
    {
        return false;
    }

    @Override
    public void func_176199_a(World worldIn, BlockPos pos, Entity entityIn)
    {
        IBlockState state = worldIn.func_180495_p(pos);
        if (state.func_177229_b(LIT) && !entityIn.func_70045_F() && entityIn instanceof EntityLivingBase && state.func_177229_b(LIT))
        {
            entityIn.func_70097_a(DamageSource.field_76372_a, 4.0f);
        }
        super.func_176199_a(worldIn, pos, entityIn);
    }

    @Override
    @Nonnull
    protected BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, LIT, LAYERS);
    }

    @Override
    public int getLightValue(IBlockState state, IBlockAccess world, BlockPos pos)
    {
        return state.func_177229_b(LIT) ? 15 : 0;
    }

    @Override
    public boolean isNormalCube(IBlockState state, IBlockAccess world, BlockPos pos)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean isSideSolid(IBlockState base_state, IBlockAccess world, BlockPos pos, EnumFacing side)
    {
        return false;
    }

    @Override
    public void getDrops(NonNullList<ItemStack> drops, IBlockAccess world, BlockPos pos, IBlockState state, int fortune)
    {
        // Drops are handled by the relevant TE (blast furnace or bloomery)
    }
}
