/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks;

import java.util.Random;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.BlockTorch;
import net.minecraft.block.SoundType;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.api.capability.size.IItemSize;
import net.dries007.tfc.api.capability.size.Size;
import net.dries007.tfc.api.capability.size.Weight;
import net.dries007.tfc.objects.blocks.property.ILightableBlock;
import net.dries007.tfc.objects.items.ItemFireStarter;
import net.dries007.tfc.objects.te.TETorchTFC;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.OreDictionaryHelper;

@ParametersAreNonnullByDefault
public class BlockTorchTFC extends BlockTorch implements IItemSize, ILightableBlock
{
    public static boolean canLight(ItemStack stack)
    {
        return stack.func_77973_b() == Item.func_150898_a(BlocksTFC.TORCH) || ItemFireStarter.canIgnite(stack);
    }

    public BlockTorchTFC()
    {
        func_180632_j(field_176227_L.func_177621_b().func_177226_a(field_176596_a, EnumFacing.UP).func_177226_a(LIT, true));
        func_149715_a(0.9375F);
        func_149675_a(true);
        func_149672_a(SoundType.field_185848_a);

        Blocks.field_150480_ab.func_180686_a(this, 5, 20);

        OreDictionaryHelper.register(this, "torch");
    }

    @Nonnull
    @Override
    public Size getSize(@Nonnull ItemStack stack)
    {
        return Size.NORMAL;
    }

    @Nonnull
    @Override
    public Weight getWeight(@Nonnull ItemStack stack)
    {
        return Weight.LIGHT;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void func_180655_c(IBlockState stateIn, World worldIn, BlockPos pos, Random rand)
    {
        if (!stateIn.func_177229_b(LIT)) return;
        super.func_180655_c(stateIn, worldIn, pos, rand);
    }

    @Override
    @Nonnull
    public IBlockState func_176203_a(int meta)
    {
        return func_176223_P().func_177226_a(LIT, meta >= 8).func_177226_a(field_176596_a, EnumFacing.func_82600_a(meta & 0b111));
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return (state.func_177229_b(LIT) ? 8 : 0) + state.func_177229_b(field_176596_a).func_176745_a();
    }

    @Override
    @Nonnull
    protected BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, field_176596_a, LIT);
    }

    @Override
    public void func_180645_a(World worldIn, BlockPos pos, IBlockState state, Random random)
    {
        TETorchTFC te = Helpers.getTE(worldIn, pos, TETorchTFC.class);
        if (te != null)
        {
            te.onRandomTick();
        }
    }

    @Override
    public boolean func_180639_a(World worldIn, BlockPos pos, IBlockState state, EntityPlayer playerIn, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ)
    {
        if (!worldIn.field_72995_K)
        {
            TETorchTFC te = Helpers.getTE(worldIn, pos, TETorchTFC.class);
            ItemStack stack = playerIn.func_184586_b(hand);
            if (te != null && BlockTorchTFC.canLight(stack))
            {
                te.light();
            }
        }
        return true;
    }

    @Override
    public int getLightValue(IBlockState state, IBlockAccess world, BlockPos pos)
    {
        return state.func_177229_b(LIT) ? super.getLightValue(state, world, pos) : 0;
    }

    @Nullable
    @Override
    public TileEntity createTileEntity(World world, IBlockState state)
    {
        return new TETorchTFC();
    }
}
