/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.agriculture;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockRenderLayer;
import net.minecraft.util.DamageSource;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.api.types.IBerryBush;
import net.dries007.tfc.objects.te.TETickCounter;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.calendar.CalendarTFC;
import net.dries007.tfc.util.calendar.ICalendar;
import net.dries007.tfc.world.classic.ClimateTFC;
import net.dries007.tfc.world.classic.chunkdata.ChunkDataTFC;

@ParametersAreNonnullByDefault
@MethodsReturnNonnullByDefault
public class BlockBerryBush extends Block
{
    public static final PropertyBool FRUITING = PropertyBool.func_177716_a("fruiting");

    private static final AxisAlignedBB SMALL_SIZE_AABB = new AxisAlignedBB(0D, 0.0D, 0, 1D, 0.25D, 1D);
    private static final AxisAlignedBB MEDIUM_SIZE_AABB = new AxisAlignedBB(0D, 0.0D, 0, 1D, 0.5D, 1D);

    private static final Map<IBerryBush, BlockBerryBush> MAP = new HashMap<>();

    public static BlockBerryBush get(IBerryBush bush)
    {
        return MAP.get(bush);
    }

    public final IBerryBush bush;

    public BlockBerryBush(IBerryBush bush)
    {
        super(Material.field_151584_j);
        this.bush = bush;
        if (MAP.put(bush, this) != null) throw new IllegalStateException("There can only be one.");
        Blocks.field_150480_ab.func_180686_a(this, 30, 60);
        func_149711_c(1.0F);
        func_149675_a(true);
        func_149672_a(SoundType.field_185850_c);
        func_180632_j(field_176227_L.func_177621_b().func_177226_a(FRUITING, false));
    }

    @Override
    public void func_176213_c(World worldIn, BlockPos pos, IBlockState state)
    {
        TETickCounter tile = Helpers.getTE(worldIn, pos, TETickCounter.class);
        if (tile != null)
        {
            tile.resetCounter();
        }
    }

    @SuppressWarnings("deprecation")
    @Override
    @Nonnull
    public IBlockState func_176203_a(int meta)
    {
        return func_176223_P().func_177226_a(FRUITING, meta == 1);
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return state.func_177229_b(FRUITING) ? 1 : 0;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149686_d(IBlockState state)
    {
        return bush.getSize() == IBerryBush.Size.LARGE;
    }

    @SuppressWarnings("deprecation")
    @Override
    public AxisAlignedBB func_180646_a(IBlockState blockState, IBlockAccess worldIn, BlockPos pos)
    {
        return field_185506_k;
    }

    @Override
    @SuppressWarnings("deprecation")
    @Nonnull
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        switch (bush.getSize())
        {
            case SMALL:
                return SMALL_SIZE_AABB;
            case MEDIUM:
                return MEDIUM_SIZE_AABB;
            default:
                return field_185505_j;
        }
    }

    @Override
    @Nonnull
    @SuppressWarnings("deprecation")
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face)
    {
        return bush.getSize() == IBerryBush.Size.LARGE ? BlockFaceShape.SOLID : BlockFaceShape.UNDEFINED;
    }

    @Override
    public void func_180645_a(World world, BlockPos pos, IBlockState state, Random random)
    {
        if (!world.field_72995_K)
        {
            TETickCounter te = Helpers.getTE(world, pos, TETickCounter.class);
            if (te != null)
            {
                float temp = ClimateTFC.getTemp(world, pos);
                float rainfall = ChunkDataTFC.getRainfall(world, pos);
                long hours = te.getTicksSinceUpdate() / ICalendar.TICKS_IN_HOUR;
                if (hours > bush.getGrowthTime() && bush.isValidForGrowth(temp, rainfall))
                {
                    if (bush.isHarvestMonth(CalendarTFC.CALENDAR_TIME.getMonthOfYear()))
                    {
                        //Fruiting
                        world.func_175656_a(pos, world.func_180495_p(pos).func_177226_a(FRUITING, true));
                    }
                    te.resetCounter();
                }
            }
        }
    }

    @Override
    public boolean func_180639_a(World worldIn, BlockPos pos, IBlockState state, EntityPlayer playerIn, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ)
    {
        if (worldIn.func_180495_p(pos).func_177229_b(FRUITING))
        {
            if (!worldIn.field_72995_K)
            {
                Helpers.spawnItemStack(worldIn, pos, bush.getFoodDrop());
                worldIn.func_175656_a(pos, worldIn.func_180495_p(pos).func_177226_a(FRUITING, false));
                TETickCounter te = Helpers.getTE(worldIn, pos, TETickCounter.class);
                if (te != null)
                {
                    te.resetCounter();
                }
            }
            return true;
        }
        return false;
    }

    @Override
    public boolean func_176196_c(World worldIn, @Nonnull BlockPos pos)
    {
        if (super.func_176196_c(worldIn, pos))
        {
            IBlockState state = worldIn.func_180495_p(pos.func_177977_b());
            return state.func_177230_c().canPlaceTorchOnTop(state, worldIn, pos);
        }
        return false;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149662_c(IBlockState state)
    {
        return false;
    }

    @Override
    public void func_180634_a(World worldIn, BlockPos pos, IBlockState state, Entity entityIn)
    {
        //Entity motion is reduced (like leaves).
        entityIn.field_70159_w *= 0.1D;
        if (entityIn.field_70181_x < 0)
        {
            entityIn.field_70181_x *= 0.1D;
        }
        entityIn.field_70179_y *= 0.1D;
        if (bush.isSpiky())
        {
            entityIn.func_70097_a(DamageSource.field_76367_g, 1.0F);
        }
    }

    @SideOnly(Side.CLIENT)
    @Override
    @Nonnull
    public BlockRenderLayer func_180664_k()
    {
        return BlockRenderLayer.CUTOUT_MIPPED;
    }

    @Override
    @Nonnull
    public BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, FRUITING);
    }

    @Override
    public boolean hasTileEntity(IBlockState state)
    {
        return true;
    }

    @Nullable
    @Override
    public TileEntity createTileEntity(World world, IBlockState state)
    {
        return new TETickCounter();
    }
}
