/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.plants;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.IGrowable;
import net.minecraft.block.properties.PropertyEnum;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.EnumSkyBlock;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import net.dries007.tfc.api.types.Plant;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.objects.blocks.property.ITallPlant;
import net.dries007.tfc.world.classic.ClimateTFC;
import net.dries007.tfc.world.classic.chunkdata.ChunkDataTFC;

import static net.dries007.tfc.world.classic.ChunkGenTFC.SALT_WATER;

import net.dries007.tfc.objects.blocks.property.ITallPlant.EnumBlockPart;
import net.minecraft.block.Block.EnumOffsetType;

@ParametersAreNonnullByDefault
public class BlockTallWaterPlantTFC extends BlockWaterPlantTFC implements IGrowable, ITallPlant
{
    private static final PropertyEnum<EnumBlockPart> PART = PropertyEnum.func_177709_a("part", EnumBlockPart.class);
    private static final Map<Plant, BlockTallWaterPlantTFC> MAP = new HashMap<>();

    public static BlockTallWaterPlantTFC get(Plant plant)
    {
        return BlockTallWaterPlantTFC.MAP.get(plant);
    }

    public BlockTallWaterPlantTFC(Plant plant)
    {
        super(plant);
        if (MAP.put(plant, this) != null) throw new IllegalStateException("There can only be one.");
    }

    @Override
    public boolean func_176473_a(World worldIn, BlockPos pos, IBlockState state, boolean isClient)
    {
        IBlockState water = plant.getWaterType();
        int i;
        //noinspection StatementWithEmptyBody
        for (i = 1; worldIn.func_180495_p(pos.func_177979_c(i)).func_177230_c() == this; ++i) ;
        if (water == SALT_WATER)
            return i < plant.getMaxHeight() && BlocksTFC.isSaltWater(worldIn.func_180495_p(pos.func_177984_a())) && func_180671_f(worldIn, pos.func_177984_a(), state);
        else
            return i < plant.getMaxHeight() && BlocksTFC.isFreshWater(worldIn.func_180495_p(pos.func_177984_a())) && func_180671_f(worldIn, pos.func_177984_a(), state);
    }

    @Override
    public boolean func_180670_a(World worldIn, Random rand, BlockPos pos, IBlockState state)
    {
        return false;
    }

    @Override
    public void func_176474_b(World worldIn, Random rand, BlockPos pos, IBlockState state)
    {
        worldIn.func_175656_a(pos.func_177984_a(), this.func_176223_P());
        IBlockState iblockstate = state.func_177226_a(AGE, 0).func_177226_a(growthStageProperty, plant.getStageForMonth()).func_177226_a(PART, getPlantPart(worldIn, pos));
        worldIn.func_175656_a(pos, iblockstate);
        iblockstate.func_189546_a(worldIn, pos.func_177984_a(), this, pos);
    }

    public void shrink(World worldIn, BlockPos pos)
    {
        worldIn.func_175656_a(pos, plant.getWaterType());
        worldIn.func_180495_p(pos).func_189546_a(worldIn, pos.func_177977_b(), this, pos);
    }

    @Override
    public void func_176208_a(World worldIn, BlockPos pos, IBlockState state, EntityPlayer player)
    {
        super.func_176208_a(worldIn, pos, state, player);
    }

    @Override
    public boolean canSustainPlant(IBlockState state, IBlockAccess world, BlockPos pos, EnumFacing direction, net.minecraftforge.common.IPlantable plantable)
    {
        IBlockState plant = plantable.getPlant(world, pos.func_177972_a(direction));

        if (plant.func_177230_c() == this)
        {
            return true;
        }
        return super.canSustainPlant(state, world, pos, direction, plantable);
    }

    @Override
    @Nonnull
    public IBlockState func_176221_a(IBlockState state, IBlockAccess worldIn, BlockPos pos)
    {
        return super.func_176221_a(state, worldIn, pos).func_177226_a(PART, getPlantPart(worldIn, pos));
    }

    @Override
    @Nonnull
    public Block.EnumOffsetType func_176218_Q()
    {
        return EnumOffsetType.XYZ;
    }

    @Override
    public void func_180650_b(World worldIn, BlockPos pos, IBlockState state, Random rand)
    {
        if (!worldIn.func_175697_a(pos, 1)) return;

        if (plant.isValidGrowthTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, pos)) && plant.isValidSunlight(Math.subtractExact(worldIn.func_175642_b(EnumSkyBlock.SKY, pos), worldIn.func_175657_ab())))
        {
            int j = state.func_177229_b(AGE);

            if (rand.nextDouble() < getGrowthRate(worldIn, pos) && net.minecraftforge.common.ForgeHooks.onCropsGrowPre(worldIn, pos.func_177984_a(), state, true))
            {
                if (j == 3 && func_176473_a(worldIn, pos, state, worldIn.field_72995_K))
                {
                    func_176474_b(worldIn, rand, pos, state);
                }
                else if (j < 3)
                {
                    worldIn.func_175656_a(pos, state.func_177226_a(AGE, j + 1).func_177226_a(PART, getPlantPart(worldIn, pos)));
                }
                net.minecraftforge.common.ForgeHooks.onCropsGrowPost(worldIn, pos, state, worldIn.func_180495_p(pos));
            }
        }
        else if (!plant.isValidGrowthTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, pos)) || !plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, pos)))
        {
            int j = state.func_177229_b(AGE);

            if (rand.nextDouble() < getGrowthRate(worldIn, pos) && net.minecraftforge.common.ForgeHooks.onCropsGrowPre(worldIn, pos, state, true))
            {
                if (j == 0 && canShrink(worldIn, pos))
                {
                    shrink(worldIn, pos);
                }
                else if (j > 0)
                {
                    worldIn.func_175656_a(pos, state.func_177226_a(AGE, j - 1).func_177226_a(PART, getPlantPart(worldIn, pos)));
                }
                net.minecraftforge.common.ForgeHooks.onCropsGrowPost(worldIn, pos, state, worldIn.func_180495_p(pos));
            }
        }

        func_176475_e(worldIn, pos, state);
    }

    @Override
    public boolean func_180671_f(World worldIn, BlockPos pos, IBlockState state)
    {
        IBlockState soil = worldIn.func_180495_p(pos.func_177977_b());

        if (worldIn.func_180495_p(pos.func_177979_c(plant.getMaxHeight())).func_177230_c() == this) return false;
        if (state.func_177230_c() == this)
        {
            return soil.func_177230_c().canSustainPlant(soil, worldIn, pos.func_177977_b(), net.minecraft.util.EnumFacing.UP, this) && plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, pos)) && plant.isValidRain(ChunkDataTFC.getRainfall(worldIn, pos));
        }
        return this.func_185514_i(soil);
    }

    @Override
    @Nonnull
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        return getTallBoundingBax(state.func_177229_b(AGE), state, source, pos);
    }

    @Override
    @Nonnull
    protected BlockStateContainer createPlantBlockState()
    {
        return new BlockStateContainer(this, AGE, growthStageProperty, PART, DAYPERIOD);
    }

    @Override
    public boolean func_176196_c(World worldIn, BlockPos pos)
    {
        return super.func_176196_c(worldIn, pos) && this.func_180671_f(worldIn, pos, worldIn.func_180495_p(pos));
    }

    private boolean canShrink(World worldIn, BlockPos pos)
    {
        return worldIn.func_180495_p(pos.func_177977_b()).func_177230_c() == this && worldIn.func_180495_p(pos.func_177984_a()).func_177230_c() != this;
    }
}
