/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.stone;

import java.util.Random;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.api.types.Rock;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class BlockPathTFC extends BlockRockVariantFallable
{
    private static final AxisAlignedBB GRASS_PATH_AABB = new AxisAlignedBB(0.0D, 0.0D, 0.0D, 1.0D, 0.9375D, 1.0D);

    public BlockPathTFC(Rock.Type type, Rock rock)
    {
        super(type, rock);
        func_149713_g(255);
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149686_d(IBlockState state)
    {
        return false;
    }

    @Override
    @SuppressWarnings("deprecation")
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        return GRASS_PATH_AABB;
    }

    @Override
    @SuppressWarnings("deprecation")
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face)
    {
        return face == EnumFacing.DOWN ? BlockFaceShape.SOLID : BlockFaceShape.UNDEFINED;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149662_c(IBlockState state)
    {
        return false;
    }

    @SideOnly(Side.CLIENT)
    public boolean func_176225_a(IBlockState blockState, IBlockAccess blockAccess, BlockPos pos, EnumFacing side)
    {
        switch (side)
        {
            case UP:
                return true;
            case NORTH:
            case SOUTH:
            case WEST:
            case EAST:
                IBlockState iblockstate = blockAccess.func_180495_p(pos.func_177972_a(side));
                Block block = iblockstate.func_177230_c();
                return !iblockstate.func_185914_p() && block != Blocks.field_150458_ak && block != Blocks.field_185774_da;
            default:
                return super.func_176225_a(blockState, blockAccess, pos, side);
        }
    }

    @Override
    public Item func_180660_a(IBlockState state, Random rand, int fortune)
    {
        return Item.func_150898_a(get(rock, Rock.Type.DIRT));
    }

    @Override
    public void func_189540_a(IBlockState state, World worldIn, BlockPos pos, Block blockIn, BlockPos fromPos)
    {
        super.func_189540_a(state, worldIn, pos, blockIn, fromPos);
        this.updateBlockState(worldIn, pos);
    }

    @Override
    public void func_176213_c(World worldIn, BlockPos pos, IBlockState state)
    {
        super.func_176213_c(worldIn, pos, state);
        this.updateBlockState(worldIn, pos);
    }

    private void updateBlockState(World worldIn, BlockPos pos)
    {
        if (worldIn.func_180495_p(pos.func_177984_a()).func_185904_a().func_76220_a())
        {
            worldIn.func_175656_a(pos, BlockRockVariant.get(rock, Rock.Type.DIRT).func_176223_P());
            for (Entity entity : worldIn.func_72839_b(null, GRASS_PATH_AABB.func_186670_a(pos)))
            {
                double initialOffset = Math.min(GRASS_PATH_AABB.func_186670_a(pos).field_72337_e - GRASS_PATH_AABB.func_186670_a(pos).field_72338_b, GRASS_PATH_AABB.func_186670_a(pos).field_72337_e - entity.func_174813_aQ().field_72338_b);
                entity.func_70634_a(entity.field_70165_t, entity.field_70163_u + initialOffset + 0.001D, entity.field_70161_v);
            }
        }
    }
}
