/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.entity.projectile;

import javax.annotation.Nonnull;

import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.monster.EntityEnderman;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.entity.projectile.EntityArrow;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.network.play.server.SPacketChangeGameState;
import net.minecraft.util.DamageSource;
import net.minecraft.util.math.MathHelper;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.World;
import net.minecraftforge.common.util.Constants;
import net.minecraftforge.fml.common.network.ByteBufUtils;
import net.minecraftforge.fml.common.registry.IEntityAdditionalSpawnData;
import net.minecraftforge.fml.common.registry.IThrowableEntity;

import io.netty.buffer.ByteBuf;

public class EntityThrownWeapon extends EntityArrow implements IThrowableEntity, IEntityAdditionalSpawnData
{
    private ItemStack weapon = ItemStack.field_190927_a;
    private int knockbackStrength = 0;

    protected EntityThrownWeapon(World world) { super(world); }

    protected EntityThrownWeapon(World world, double x, double y, double z) { super(world, x, y, z); }

    protected EntityThrownWeapon(World world, EntityLivingBase shooter) { super(world, shooter); }

    @Override
    public Entity getThrower()
    {
        return this.field_70250_c;
    }

    @Override
    public void setThrower(Entity entity)
    {
        this.field_70250_c = entity;
    }

    @Override
    public void writeSpawnData(ByteBuf buffer)
    {
        ByteBufUtils.writeItemStack(buffer, this.weapon);
    }

    @Override
    public void readSpawnData(ByteBuf additionalData)
    {
        setWeapon(ByteBufUtils.readItemStack(additionalData));
    }

    public ItemStack getWeapon()
    {
        return this.weapon;
    }

    public void setWeapon(ItemStack stack) { this.weapon = stack.func_77946_l(); }

    @Override
    protected void func_184549_a(@Nonnull RayTraceResult raytraceResultIn)
    {
        Entity entity = raytraceResultIn.field_72308_g;

        // Damage item
        if (getThrower() instanceof EntityLivingBase)
        {
            EntityLivingBase thrower = (EntityLivingBase) getThrower();
            int damageAmount = 1;
            if (entity != null)
            {
                damageAmount += 1 + thrower.func_70681_au().nextInt(4);
            }
            weapon.func_77972_a(damageAmount, thrower);
        }

        if (entity != null)
        {
            ItemStack weapon = this.getWeapon();
            float f = MathHelper.func_76133_a(this.field_70159_w * this.field_70159_w + this.field_70181_x * this.field_70181_x + this.field_70179_y * this.field_70179_y);
            float finalDamage = f * (float) this.func_70242_d();

            if (this.func_70241_g())
            {
                finalDamage *= 2.0f;
            }

            DamageSource damagesource;

            if (this.field_70250_c == null)
            {
                //TODO custom damage sources?
                damagesource = DamageSource.func_76353_a(this, this);
            }
            else
            {
                damagesource = DamageSource.func_76353_a(this, this.field_70250_c);
            }

            if (this.func_70027_ad() && !(entity instanceof EntityEnderman))
            {
                entity.func_70015_d(5);
            }

            if (entity.func_70097_a(damagesource, finalDamage))
            {
                if (entity instanceof EntityLivingBase)
                {
                    EntityLivingBase entitylivingbase = (EntityLivingBase) entity;

                    if (this.knockbackStrength > 0)
                    {
                        float f1 = MathHelper.func_76133_a(this.field_70159_w * this.field_70159_w + this.field_70179_y * this.field_70179_y);

                        if (f1 > 0.0F)
                        {
                            entitylivingbase.func_70024_g(this.field_70159_w * (double) this.knockbackStrength * 0.6000000238418579D / (double) f1, 0.1D, this.field_70179_y * (double) this.knockbackStrength * 0.6000000238418579D / (double) f1);
                        }
                    }

                    this.func_184548_a(entitylivingbase);

                    if (this.field_70250_c != null && entitylivingbase != this.field_70250_c && entitylivingbase instanceof EntityPlayer && this.field_70250_c instanceof EntityPlayerMP)
                    {
                        ((EntityPlayerMP) this.field_70250_c).field_71135_a.func_147359_a(new SPacketChangeGameState(6, 0.0F));
                    }
                }

                //TODO custom sound for thrown weapon hit?
                this.func_184185_a(SoundEvents.field_187731_t, 1.0F, 1.2F / (this.field_70146_Z.nextFloat() * 0.2F + 0.9F));
                this.func_70099_a(weapon, 0.1F);
                this.func_70106_y();
            }
            else
            {
                this.field_70159_w *= -0.10000000149011612D;
                this.field_70181_x *= -0.10000000149011612D;
                this.field_70179_y *= -0.10000000149011612D;
                this.field_70177_z += 180.0F;
                this.field_70126_B += 180.0F;

                if (!this.field_70170_p.field_72995_K && this.field_70159_w * this.field_70159_w + this.field_70181_x * this.field_70181_x + this.field_70179_y * this.field_70179_y < 0.0010000000474974513D)
                {
                    this.func_70099_a(weapon, 0.1F);
                    this.func_70106_y();
                }
            }
        }
        else
        {
            super.func_184549_a(raytraceResultIn);
        }
    }

    @Override
    public void func_70014_b(NBTTagCompound compound)
    {
        NBTTagList tag = new NBTTagList();
        tag.func_74742_a(weapon.serializeNBT());
        compound.func_74782_a("weapon", tag);

        super.func_70014_b(compound);
    }

    @Override
    public void func_70037_a(NBTTagCompound compound)
    {
        NBTTagList tag = compound.func_150295_c("weapon", Constants.NBT.TAG_COMPOUND);
        weapon = tag.func_74745_c() > 0 ? new ItemStack(tag.func_150305_b(0)) : ItemStack.field_190927_a;
        super.func_70037_a(compound);
    }

    @Override
    @Nonnull
    protected ItemStack func_184550_j()
    {
        return this.weapon;
    }

    public void func_70240_a(int knockbackStrength)
    {
        this.knockbackStrength = knockbackStrength;
    }
}
