/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.items.itemblock;

import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.advancements.CriteriaTriggers;
import net.minecraft.block.BlockLiquid;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.stats.StatList;
import net.minecraft.util.ActionResult;
import net.minecraft.util.EnumActionResult;
import net.minecraft.util.EnumHand;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.World;

import net.dries007.tfc.objects.blocks.plants.BlockFloatingWaterTFC;

@ParametersAreNonnullByDefault
public class ItemBlockFloatingWaterTFC extends ItemBlockPlant
{
    protected BlockFloatingWaterTFC block;

    public ItemBlockFloatingWaterTFC(BlockFloatingWaterTFC block)
    {
        super(block);
        this.block = block;
    }

    @Override
    @Nonnull
    public ActionResult<ItemStack> func_77659_a(World worldIn, EntityPlayer playerIn, EnumHand handIn)
    {
        ItemStack itemstack = playerIn.func_184586_b(handIn);
        RayTraceResult raytraceresult = this.func_77621_a(worldIn, playerIn, true);

        //noinspection ConstantConditions
        if (raytraceresult == null)
        {
            return new ActionResult<>(EnumActionResult.PASS, itemstack);
        }
        else
        {
            if (raytraceresult.field_72313_a == RayTraceResult.Type.BLOCK)
            {
                BlockPos blockpos = raytraceresult.func_178782_a();

                if (!worldIn.func_175660_a(playerIn, blockpos) || !playerIn.func_175151_a(blockpos.func_177972_a(raytraceresult.field_178784_b), raytraceresult.field_178784_b, itemstack))
                {
                    return new ActionResult<>(EnumActionResult.FAIL, itemstack);
                }

                BlockPos blockpos1 = blockpos.func_177984_a();
                IBlockState iblockstate = worldIn.func_180495_p(blockpos);

                if (iblockstate.func_185904_a() == Material.field_151586_h && iblockstate.func_177229_b(BlockLiquid.field_176367_b) == 0 && worldIn.func_175623_d(blockpos1) && iblockstate == block.getPlant().getWaterType())
                {
                    // special case for handling block placement with water lilies
                    net.minecraftforge.common.util.BlockSnapshot blocksnapshot = net.minecraftforge.common.util.BlockSnapshot.getBlockSnapshot(worldIn, blockpos1);
                    worldIn.func_175656_a(blockpos1, block.func_176223_P());
                    if (net.minecraftforge.event.ForgeEventFactory.onPlayerBlockPlace(playerIn, blocksnapshot, net.minecraft.util.EnumFacing.UP, handIn).isCanceled())
                    {
                        blocksnapshot.restore(true, false);
                        return new ActionResult<>(EnumActionResult.FAIL, itemstack);
                    }

                    worldIn.func_180501_a(blockpos1, block.func_176223_P(), 11);

                    if (playerIn instanceof EntityPlayerMP)
                    {
                        CriteriaTriggers.field_193137_x.func_193173_a((EntityPlayerMP) playerIn, blockpos1, itemstack);
                    }

                    if (!playerIn.field_71075_bZ.field_75098_d)
                    {
                        itemstack.func_190918_g(1);
                    }

                    //noinspection ConstantConditions
                    playerIn.func_71029_a(StatList.func_188057_b(this));
                    worldIn.func_184133_a(playerIn, blockpos, SoundEvents.field_187916_gp, SoundCategory.BLOCKS, 1.0F, 1.0F);
                    return new ActionResult<>(EnumActionResult.SUCCESS, itemstack);
                }
            }

            return new ActionResult<>(EnumActionResult.FAIL, itemstack);
        }
    }
}
