/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.items.metal;

import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3d;
import net.minecraft.world.World;

import net.dries007.tfc.api.types.Metal;
import net.dries007.tfc.api.util.IPlaceableItem;
import net.dries007.tfc.objects.blocks.metal.BlockMetalSheet;
import net.dries007.tfc.objects.te.TEMetalSheet;
import net.dries007.tfc.util.Helpers;

public class ItemMetalSheet extends ItemMetal implements IPlaceableItem
{
    public ItemMetalSheet(Metal metal, Metal.ItemType type)
    {
        super(metal, type);
    }

    @Override
    public boolean placeItemInWorld(World world, BlockPos pos, ItemStack stack, EntityPlayer player, EnumFacing facing, Vec3d hitVec)
    {
        if (world.func_180495_p(pos).func_185915_l() && stack.func_77973_b() instanceof ItemMetalSheet)
        {
            ItemMetalSheet sheet = (ItemMetalSheet) stack.func_77973_b();
            BlockPos posAt = pos.func_177972_a(facing);
            IBlockState stateAt = world.func_180495_p(posAt);

            if (stateAt.func_177230_c() instanceof BlockMetalSheet)
            {
                // Existing sheet block
                Metal metal = ((BlockMetalSheet) stateAt.func_177230_c()).getMetal();
                if (metal == sheet.metal)
                {
                    return placeSheet(world, posAt, facing);
                }
            }
            else if (stateAt.func_177230_c().func_176200_f(world, posAt))
            {
                // Place a new block
                if (!world.field_72995_K)
                {
                    world.func_175656_a(posAt, BlockMetalSheet.get(sheet.metal).func_176223_P().func_177226_a(BlockMetalSheet.FACE_PROPERTIES[facing.func_176745_a()], true));
                    placeSheet(world, posAt, facing);
                }
                return true;
            }
        }
        return false;
    }

    private boolean placeSheet(World world, BlockPos pos, EnumFacing facing)
    {
        TEMetalSheet tile = Helpers.getTE(world, pos, TEMetalSheet.class);
        if (tile != null && !tile.getFace(facing))
        {
            if (!world.field_72995_K)
            {
                tile.setFace(facing, true);
                world.func_184133_a(null, pos.func_177972_a(facing), SoundEvents.field_187772_dn, SoundCategory.BLOCKS, 1.0f, 1.0f);
            }
            return true;
        }
        return false;
    }
}
