/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.te;

import java.util.ArrayList;
import java.util.List;
import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.inventory.InventoryHelper;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.EntitySelectors;
import net.minecraft.util.ITickable;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.common.util.Constants;

import net.dries007.tfc.ConfigTFC;
import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.api.capability.heat.CapabilityItemHeat;
import net.dries007.tfc.api.capability.heat.IItemHeat;
import net.dries007.tfc.api.types.Metal;
import net.dries007.tfc.api.util.IHeatConsumerBlock;
import net.dries007.tfc.api.util.IMetalObject;
import net.dries007.tfc.objects.blocks.BlockMolten;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.OreDictionaryHelper;
import net.dries007.tfc.util.fuel.Fuel;
import net.dries007.tfc.util.fuel.FuelManager;

import static net.dries007.tfc.api.capability.heat.CapabilityItemHeat.MAX_TEMPERATURE;
import static net.dries007.tfc.objects.blocks.property.ILightableBlock.LIT;

@ParametersAreNonnullByDefault
public class TEBlastFurnace extends TEInventory implements ITickable, ITileFields
{
    public static final int SLOT_TUYERE = 0;
    public static final int FIELD_TEMPERATURE = 0, FIELD_ORE = 1, FIELD_FUEL = 2, FIELD_MELT = 3, FIELD_ORE_UNITS = 4;

    private List<ItemStack> oreStacks = new ArrayList<>();
    private List<ItemStack> fuelStacks = new ArrayList<>();

    private int maxFuel = 0, maxOre = 0, delayTimer = 0, meltAmount = 0;
    private long burnTicksLeft = 0, airTicks = 0;
    private int fuelCount = 0, oreCount = 0, oreUnits; // Used to show on client's GUI how much ore/fuel TE has

    private int temperature = 0;
    private float burnTemperature = 0;

    public TEBlastFurnace()
    {
        super(1);
    }

    @Override
    public int getSlotLimit(int slot)
    {
        return 1;
    }

    @Override
    public boolean isItemValid(int slot, ItemStack stack)
    {
        return OreDictionaryHelper.doesStackMatchOre(stack, "tuyere");
    }

    @Override
    public void func_145839_a(NBTTagCompound nbt)
    {
        oreStacks.clear();
        NBTTagList ores = nbt.func_150295_c("ores", Constants.NBT.TAG_COMPOUND);
        for (int i = 0; i < ores.func_74745_c(); i++)
        {
            oreStacks.add(new ItemStack(ores.func_150305_b(i)));
        }

        fuelStacks.clear();
        NBTTagList fuels = nbt.func_150295_c("fuels", Constants.NBT.TAG_COMPOUND);
        for (int i = 0; i < ores.func_74745_c(); i++)
        {
            fuelStacks.add(new ItemStack(fuels.func_150305_b(i)));
        }
        burnTicksLeft = nbt.func_74763_f("burnTicksLeft");
        airTicks = nbt.func_74763_f("airTicks");
        burnTemperature = nbt.func_74760_g("burnTemperature");
        temperature = nbt.func_74762_e("temperature");
        meltAmount = nbt.func_74762_e("meltAmount");
        super.func_145839_a(nbt);
    }

    @Override
    @Nonnull
    public NBTTagCompound func_189515_b(NBTTagCompound nbt)
    {
        NBTTagList ores = new NBTTagList();
        for (ItemStack stack : oreStacks)
        {
            ores.func_74742_a(stack.serializeNBT());
        }
        nbt.func_74782_a("ores", ores);
        NBTTagList fuels = new NBTTagList();
        for (ItemStack stack : fuelStacks)
        {
            fuels.func_74742_a(stack.serializeNBT());
        }
        nbt.func_74782_a("fuels", ores);
        nbt.func_74772_a("burnTicksLeft", burnTicksLeft);
        nbt.func_74772_a("airTicks", airTicks);
        nbt.func_74776_a("burnTemperature", burnTemperature);
        nbt.func_74768_a("temperature", temperature);
        nbt.func_74768_a("meltAmount", meltAmount);
        return super.func_189515_b(nbt);
    }

    @Override
    public void onBreakBlock(World worldIn, BlockPos pos)
    {
        //Dump everything in world
        for (int i = 1; i < 6; i++)
        {
            if (field_145850_b.func_180495_p(pos.func_177981_b(i)).func_177230_c() == BlocksTFC.MOLTEN)
            {
                field_145850_b.func_175698_g(pos.func_177981_b(i));
            }
        }
        for (ItemStack stack : oreStacks)
        {
            InventoryHelper.func_180173_a(field_145850_b, pos.func_177978_c().func_177958_n(), pos.func_177956_o(), pos.func_177978_c().func_177952_p(), stack);
        }
        for (ItemStack stack : fuelStacks)
        {
            InventoryHelper.func_180173_a(field_145850_b, pos.func_177978_c().func_177958_n(), pos.func_177956_o(), pos.func_177978_c().func_177952_p(), stack);
        }
        super.onBreakBlock(field_145850_b, pos);
    }

    public boolean canIgnite()
    {
        if (!field_145850_b.field_72995_K)
        {
            return !fuelStacks.isEmpty() && !oreStacks.isEmpty();
        }
        return false;
    }

    @Override
    public int getFieldCount()
    {
        return 5;
    }

    @Override
    public void setField(int index, int value)
    {
        switch (index)
        {
            case FIELD_TEMPERATURE:
                temperature = value;
                return;
            case FIELD_ORE:
                oreCount = value;
                return;
            case FIELD_FUEL:
                fuelCount = value;
                return;
            case FIELD_MELT:
                meltAmount = value;
                return;
            case FIELD_ORE_UNITS:
                oreUnits = value;
                return;
        }
        TerraFirmaCraft.getLog().warn("Illegal field id {} in TEBlastFurnace#setField", index);
    }

    @Override
    public int getField(int index)
    {
        switch (index)
        {
            case FIELD_TEMPERATURE:
                return temperature;
            case FIELD_ORE:
                return oreCount;
            case FIELD_FUEL:
                return fuelCount;
            case FIELD_MELT:
                return meltAmount;
            case FIELD_ORE_UNITS:
                return oreUnits;
        }
        TerraFirmaCraft.getLog().warn("Illegal field id {} in TEBlastFurnace#getField", index);
        return 0;
    }

    @Override
    public void func_73660_a()
    {
        if (!field_145850_b.field_72995_K)
        {
            IBlockState state = field_145850_b.func_180495_p(field_174879_c);

            if (--delayTimer <= 0)
            {
                delayTimer = 20;
                // Update multiblock status
                int newMaxItems = BlocksTFC.BLAST_FURNACE.getChimneyLevels(field_145850_b, field_174879_c) * 4;
                maxFuel = newMaxItems;
                maxOre = newMaxItems;
                while (maxOre < oreStacks.size())
                {
                    //Structure lost one or more chimney levels
                    InventoryHelper.func_180173_a(field_145850_b, field_174879_c.func_177978_c().func_177958_n(), field_174879_c.func_177956_o(), field_174879_c.func_177978_c().func_177952_p(), oreStacks.get(0));
                    oreStacks.remove(0);
                }
                while (maxFuel < fuelStacks.size())
                {
                    InventoryHelper.func_180173_a(field_145850_b, field_174879_c.func_177978_c().func_177958_n(), field_174879_c.func_177978_c().func_177956_o(), field_174879_c.func_177978_c().func_177952_p(), fuelStacks.get(0));
                    fuelStacks.remove(0);
                }
                if (newMaxItems <= 0)
                {
                    //Structure became compromised
                    field_145850_b.func_175655_b(field_174879_c, true);
                    return;
                }
                addItemsFromWorld();
                updateSlagBlock(state.func_177229_b(LIT));

                oreCount = oreStacks.size();
                oreUnits = oreStacks.stream().mapToInt(stack -> {
                    if (stack.func_77973_b() instanceof IMetalObject)
                    {
                        return ((IMetalObject) stack.func_77973_b()).getSmeltAmount(stack);
                    }
                    return 1;
                }).sum();
                fuelCount = fuelStacks.size();
            }
            if (meltAmount > 0)
            {
                //Move already molten liquid metal to the crucible.
                //This makes the effect of slow(not so much) filling up the crucible.
                TECrucible te = Helpers.getTE(field_145850_b, field_174879_c.func_177977_b(), TECrucible.class);
                if (te != null)
                {
                    te.addMetal(Metal.PIG_IRON, 1);
                    meltAmount -= 1;
                }
            }
            if (state.func_177229_b(LIT))
            {
                // Update bellows air
                if (--airTicks <= 0)
                {
                    airTicks = 0;
                }

                if (--burnTicksLeft <= 0)
                {
                    if (!fuelStacks.isEmpty())
                    {
                        ItemStack fuelStack = fuelStacks.get(0);
                        fuelStacks.remove(0);
                        Fuel fuel = FuelManager.getFuel(fuelStack);
                        burnTicksLeft = fuel.getAmount();
                        burnTemperature = fuel.getTemperature();
                        this.func_70296_d();
                    }
                    else
                    {
                        burnTemperature = 0;
                    }
                }

                if (temperature > 0 || burnTemperature > 0)
                {
                    float targetTemperature = Math.min(MAX_TEMPERATURE, burnTemperature + airTicks);
                    if (temperature < targetTemperature)
                    {
                        // Modifier for heating = 2x for bellows
                        temperature += (airTicks > 0 ? 2 : 1) * ConfigTFC.GENERAL.temperatureModifierHeating;
                    }
                    else if (temperature > targetTemperature)
                    {
                        // Modifier for cooling = 0.5x for bellows
                        temperature -= (airTicks > 0 ? 0.5 : 1) * ConfigTFC.GENERAL.temperatureModifierHeating;
                    }
                    // Provide heat to blocks that are one block bellow AKA crucible
                    Block blockCrucible = field_145850_b.func_180495_p(field_174879_c.func_177977_b()).func_177230_c();
                    if (blockCrucible instanceof IHeatConsumerBlock)
                    {
                        ((IHeatConsumerBlock) blockCrucible).acceptHeat(field_145850_b, field_174879_c.func_177977_b(), temperature);
                    }
                    oreStacks.removeIf(stack ->
                    {
                        IItemHeat cap = stack.getCapability(CapabilityItemHeat.ITEM_HEAT_CAPABILITY, null);
                        if (cap != null)
                        {
                            // Update temperature of item
                            float itemTemp = cap.getTemperature();
                            if (temperature > itemTemp)
                            {
                                CapabilityItemHeat.addTemp(cap);
                            }
                            if (cap.isMolten())
                            {
                                convertToMolten(stack);
                                ItemStack tuyereStack = inventory.getStackInSlot(0);
                                if (!tuyereStack.func_190926_b())
                                {
                                    Helpers.damageItem(tuyereStack);
                                }
                                return true;
                            }
                        }
                        return false;
                    });
                }
                if (temperature <= 0 && burnTemperature <= 0)
                {
                    temperature = 0;
                    field_145850_b.func_175656_a(field_174879_c, state.func_177226_a(LIT, false));
                }
            }
        }
    }

    public void debug()
    {
        TerraFirmaCraft.getLog().debug("Debugging Blast Furnace:");
        TerraFirmaCraft.getLog().debug("Temp {} | Burn Temp {} | Fuel Ticks {}", temperature, burnTemperature, burnTicksLeft);
        TerraFirmaCraft.getLog().debug("Burning? {}", field_145850_b.func_180495_p(field_174879_c).func_177229_b(LIT));
        int i = 0;
        for (ItemStack item : oreStacks)
        {
            TerraFirmaCraft.getLog().debug("Slot: {} - NBT: {}", i, item.serializeNBT().toString());
        }
    }

    /**
     * Passed from BlockBlastFurnace's IBellowsConsumerBlock
     *
     * @param airAmount the air amount
     */
    public void onAirIntake(int airAmount)
    {
        ItemStack stack = inventory.getStackInSlot(SLOT_TUYERE);
        if (!stack.func_190926_b() && burnTicksLeft > 0)
        {
            airTicks += airAmount;
            if (airTicks > 600)
            {
                airTicks = 600;
            }
        }
    }

    /**
     * Melts stacks
     */
    private void convertToMolten(ItemStack stack)
    {
        if (!stack.func_190926_b() && stack.func_77973_b() instanceof IMetalObject)
        {
            IMetalObject metal = (IMetalObject) stack.func_77973_b();
            meltAmount += metal.getSmeltAmount(stack);
        }
    }

    private void addItemsFromWorld()
    {
        EntityItem fluxEntity = null, oreEntity = null;
        for (EntityItem entityItem : field_145850_b.func_175647_a(EntityItem.class, new AxisAlignedBB(field_174879_c.func_177984_a(), field_174879_c.func_177984_a().func_177982_a(1, 5, 1)), EntitySelectors.field_94557_a))
        {
            ItemStack stack = entityItem.func_92059_d();
            if (FuelManager.isItemFuel(stack))
            {
                // Add fuel
                while (maxFuel > fuelStacks.size())
                {
                    func_70296_d();
                    fuelStacks.add(stack.func_77979_a(1));
                    if (stack.func_190916_E() <= 0)
                    {
                        entityItem.func_70106_y();
                        break;
                    }
                }
            }
            else if (stack.func_77973_b() instanceof IMetalObject)
            {
                IMetalObject metalItem = (IMetalObject) stack.func_77973_b();
                Metal metal = metalItem.getMetal(stack);
                if (metal == Metal.WROUGHT_IRON || metal == Metal.PIG_IRON)
                {
                    oreEntity = entityItem;
                }
            }
            else if (OreDictionaryHelper.doesStackMatchOre(stack, "dustFlux"))
            {
                fluxEntity = entityItem;
            }
        }

        // Add each ore consuming flux
        while (maxOre > oreStacks.size())
        {
            if (fluxEntity == null || oreEntity == null)
            {
                break;
            }
            func_70296_d();

            ItemStack flux = fluxEntity.func_92059_d();
            flux.func_190918_g(1);

            ItemStack ore = oreEntity.func_92059_d();
            oreStacks.add(ore.func_77979_a(1));

            if (flux.func_190916_E() <= 0)
            {
                fluxEntity.func_70106_y();
                fluxEntity = null;
            }

            if (ore.func_190916_E() <= 0)
            {
                oreEntity.func_70106_y();
                oreEntity = null;
            }
        }
    }

    private void updateSlagBlock(boolean cooking)
    {
        int slag = fuelStacks.size() + oreStacks.size();
        //If there's at least one item, show one layer so player knows that it is holding stacks
        int slagLayers = slag == 1 ? 1 : slag / 2;
        for (int i = 1; i < 6; i++)
        {
            if (slagLayers > 0)
            {
                if (slagLayers >= 4)
                {
                    slagLayers -= 4;
                    field_145850_b.func_175656_a(field_174879_c.func_177981_b(i), BlocksTFC.MOLTEN.func_176223_P().func_177226_a(LIT, cooking).func_177226_a(BlockMolten.LAYERS, 4));
                }
                else
                {
                    field_145850_b.func_175656_a(field_174879_c.func_177981_b(i), BlocksTFC.MOLTEN.func_176223_P().func_177226_a(LIT, cooking).func_177226_a(BlockMolten.LAYERS, slagLayers));
                    slagLayers = 0;
                }
            }
            else
            {
                //Remove any surplus slag(ie: after cooking/structure became compromised)
                if (field_145850_b.func_180495_p(field_174879_c.func_177981_b(i)).func_177230_c() == BlocksTFC.MOLTEN)
                {
                    field_145850_b.func_175698_g(field_174879_c.func_177981_b(i));
                }
            }
        }
    }
}
