/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.util;

import net.minecraft.block.state.IBlockState;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

import net.dries007.tfc.ConfigTFC;
import net.dries007.tfc.Constants;
import net.dries007.tfc.client.TFCSoundEvents;
import net.dries007.tfc.objects.blocks.stone.BlockRockVariantFallable;
import net.dries007.tfc.objects.blocks.wood.BlockSupport;

// For raw stone, because it collapses
public interface ICollapsableBlock
{
    // Used to get cobblestone variant of this collapsable raw stone
    BlockRockVariantFallable getFallingVariant();

    // Check if this pos is collapsable
    default boolean canCollapse(World world, BlockPos pos)
    {
        return world.func_180495_p(pos.func_177977_b()).func_185904_a().func_76222_j();
    }

    /**
     * Check an area of blocks for collapsing mechanics
     *
     * @param worldIn the worldObj this block is in
     * @param pos     the BlockPos this block has been mined from
     */
    default void checkCollapsingArea(World worldIn, BlockPos pos)
    {
        if (worldIn.field_72995_K || !worldIn.func_175707_a(pos.func_177982_a(-32, -32, -32), pos.func_177982_a(32, 32, 32)))
            return; //First, let's check if this area is loaded and is on server
        if (Constants.RNG.nextDouble() < ConfigTFC.GENERAL.collapseChance) //Then, we check rng if a collapse should trigger
        {
            //Rng the radius
            int radX = (Constants.RNG.nextInt(5) + 4) / 2;
            int radY = (Constants.RNG.nextInt(3) + 2) / 2;
            int radZ = (Constants.RNG.nextInt(5) + 4) / 2;
            for (BlockPos checking : BlockSupport.getAllUnsupportedBlocksIn(worldIn, pos.func_177982_a(-radX, -radY, -radZ), pos.func_177982_a(radX, radY, radZ))) //9x5x9 max
            {
                //Check the area for a block collapse!
                if (worldIn.func_180495_p(checking).func_177230_c() instanceof ICollapsableBlock)
                {
                    ICollapsableBlock block = (ICollapsableBlock) worldIn.func_180495_p(checking).func_177230_c();
                    if (block.canCollapse(worldIn, checking)) //Still needs this to check if this can collapse without support(ie: no blocks below)
                    {
                        //Trigger collapse!
                        block.collapseArea(worldIn, checking);
                        worldIn.func_184133_a(null, pos, TFCSoundEvents.ROCK_SLIDE_LONG, SoundCategory.BLOCKS, 1.0F, 1.0F);
                        return; //Don't need to check other blocks
                    }
                }
            }
        }

    }

    /**
     * Collapse an area of ICollapsableBlocks!
     *
     * @param world       the worldObj
     * @param centerPoint the center of this area
     */
    default void collapseArea(World world, BlockPos centerPoint)
    {
        int radiusH = (world.field_73012_v.nextInt(31) + 5) / 2; //5-36
        for (BlockPos cavein : BlockSupport.getAllUnsupportedBlocksIn(world, centerPoint.func_177982_a(-radiusH, -4, -radiusH), centerPoint.func_177982_a(radiusH, 1, radiusH)))
        {
            IBlockState st = world.func_180495_p(cavein);
            if (st.func_177230_c() instanceof ICollapsableBlock
                && ((ICollapsableBlock) st.func_177230_c()).canCollapse(world, cavein))
            {
                double distSqrd =
                    Math.pow(centerPoint.func_177958_n() - cavein.func_177958_n(), 2)
                        + Math.pow(centerPoint.func_177956_o() - cavein.func_177956_o(), 2)
                        + Math.pow(centerPoint.func_177952_p() - cavein.func_177952_p(), 2);
                double chance = ConfigTFC.GENERAL.propogateCollapseChance - 0.01 * Math.sqrt(distSqrd);
                if (Constants.RNG.nextDouble() < chance)
                {
                    BlockRockVariantFallable fallingBlock = ((ICollapsableBlock) st.func_177230_c()).getFallingVariant();
                    world.func_175656_a(cavein, fallingBlock.func_176223_P());
                    fallingBlock.checkFalling(world, cavein, world.func_180495_p(cavein));
                }
            }
        }
    }
}
