/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.util.calendar;

import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.world.World;
import net.minecraft.world.storage.MapStorage;
import net.minecraft.world.storage.WorldSavedData;

import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;

@ParametersAreNonnullByDefault
public class CalendarWorldData extends WorldSavedData
{
    private static final String NAME = MOD_ID + ":calendar";

    @Nonnull
    public static CalendarWorldData get(@Nonnull World world)
    {
        MapStorage mapStorage = world.func_175693_T();
        if (mapStorage != null)
        {
            CalendarWorldData data = (CalendarWorldData) mapStorage.func_75742_a(CalendarWorldData.class, NAME);
            if (data == null)
            {
                // Unable to load data, so assign default values
                data = new CalendarWorldData();
                data.func_76185_a();
                mapStorage.func_75745_a(NAME, data);
            }
            return data;
        }
        throw new IllegalStateException("Map Storage is NULL!");
    }

    CalendarTFC instance;

    @SuppressWarnings("WeakerAccess")
    public CalendarWorldData()
    {
        super(NAME);
        instance = new CalendarTFC();
    }

    @SuppressWarnings("unused")
    public CalendarWorldData(String name)
    {
        super(name);
        instance = new CalendarTFC();
    }

    @Override
    public void func_76184_a(NBTTagCompound nbt)
    {
        instance.deserializeNBT(nbt.func_74775_l("calendar"));
    }

    @Override
    @Nonnull
    public NBTTagCompound func_189551_b(NBTTagCompound nbt)
    {
        nbt.func_74782_a("calendar", instance.serializeNBT());
        return nbt;
    }
}
