/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.world.classic.biomes;

import java.util.ArrayList;
import java.util.List;

import net.minecraft.world.biome.Biome;
import net.minecraftforge.common.BiomeDictionary;
import net.minecraftforge.event.RegistryEvent;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.registry.GameRegistry;
import net.minecraftforge.registries.IForgeRegistry;

import net.dries007.tfc.util.Helpers;

import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;
import static net.dries007.tfc.api.util.TFCConstants.MOD_NAME;

@Mod.EventBusSubscriber(modid = MOD_ID)
@GameRegistry.ObjectHolder(MOD_ID)
public final class BiomesTFC
{
    public static final BiomeTFC OCEAN = Helpers.getNull();
    public static final BiomeTFC RIVER = Helpers.getNull();
    public static final BiomeTFC BEACH = Helpers.getNull();
    public static final BiomeTFC GRAVEL_BEACH = Helpers.getNull();
    public static final BiomeTFC HIGH_HILLS = Helpers.getNull();
    public static final BiomeTFC PLAINS = Helpers.getNull();
    public static final BiomeTFC SWAMPLAND = Helpers.getNull();
    public static final BiomeTFC HIGH_HILLS_EDGE = Helpers.getNull();
    public static final BiomeTFC ROLLING_HILLS = Helpers.getNull();
    public static final BiomeTFC MOUNTAINS = Helpers.getNull();
    public static final BiomeTFC MOUNTAINS_EDGE = Helpers.getNull();
    public static final BiomeTFC HIGH_PLAINS = Helpers.getNull();
    public static final BiomeTFC DEEP_OCEAN = Helpers.getNull();
    public static final BiomeTFC LAKE = Helpers.getNull();

    private static final List<Biome> SPAWN_BIOMES = new ArrayList<>();
    private static final List<Biome> WORLD_GEN_BIOMES = new ArrayList<>();

    @SubscribeEvent
    public static void registerBiomes(RegistryEvent.Register<Biome> event)
    {
        IForgeRegistry<Biome> r = event.getRegistry();

        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Ocean").func_185398_c(-3.6f).func_185400_d(-2.69999f)), false, true, BiomeDictionary.Type.OCEAN, BiomeDictionary.Type.WET, BiomeDictionary.Type.WATER);
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " River").func_185398_c(-3.2f).func_185400_d(-3f)), false, false, BiomeDictionary.Type.RIVER, BiomeDictionary.Type.WET, BiomeDictionary.Type.WATER);
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Beach").func_185398_c(-2.69f).func_185400_d(-2.68f)), false, false, BiomeDictionary.Type.BEACH);
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Gravel Beach").func_185398_c(-2.69f).func_185400_d(-2.68f).func_185399_a("tfc:beach")), false, false, BiomeDictionary.Type.BEACH);
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " High Hills").func_185398_c(-1.9000001f).func_185400_d(-1.1f)), false, true, BiomeDictionary.Type.HILLS);
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Plains").func_185398_c(-2.6000001f).func_185400_d(-2.54f)).setSpawnBiome(), true, true, BiomeDictionary.Type.PLAINS);
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Swampland").func_185398_c(-2.8f).func_185400_d(-2.6000001f), 16, 45).setSpawnBiome(), true, true, BiomeDictionary.Type.SWAMP);
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " High Hills Edge").func_185398_c(-2.5f).func_185400_d(-2.3f).func_185399_a("tfc:high_hills")), false, false, BiomeDictionary.Type.HILLS, BiomeDictionary.Type.PLAINS);
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Rolling Hills").func_185398_c(-2.6000001f).func_185400_d(-2.3f)).setSpawnBiome(), true, true, BiomeDictionary.Type.HILLS);
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Mountains").func_185398_c(-1.9000001f).func_185400_d(-1.1f)).setSpawnBiome(), true, true, BiomeDictionary.Type.MOUNTAIN);
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Mountains Edge").func_185398_c(-2.3f).func_185400_d(-1.9000001f).func_185399_a("tfc:mountains")), false, false, BiomeDictionary.Type.MOUNTAIN);
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " High Plains").func_185398_c(-2.3f).func_185400_d(-2.27f)).setSpawnBiome(), true, true, BiomeDictionary.Type.HILLS, BiomeDictionary.Type.PLAINS);
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Deep Ocean").func_185398_c(-4.2f).func_185400_d(-2.69999f).func_185399_a("tfc:ocean")), false, false, BiomeDictionary.Type.OCEAN, BiomeDictionary.Type.WET, BiomeDictionary.Type.WATER);
        register(r, new BiomeTFC(new Biome.BiomeProperties(MOD_NAME + " Lake").func_185398_c(-3.2f).func_185400_d(-2.6990001f).func_185399_a("tfc:ocean"), 4, 5), false, false, BiomeDictionary.Type.RIVER, BiomeDictionary.Type.WET, BiomeDictionary.Type.WATER);
    }

    public static boolean isOceanicBiome(int id)
    {
        return Biome.func_185362_a(OCEAN) == id || Biome.func_185362_a(DEEP_OCEAN) == id;
    }

    public static boolean isMountainBiome(int id)
    {
        return Biome.func_185362_a(MOUNTAINS) == id || Biome.func_185362_a(MOUNTAINS_EDGE) == id;
    }

    public static boolean isBeachBiome(int id)
    {
        return Biome.func_185362_a(BEACH) == id || Biome.func_185362_a(GRAVEL_BEACH) == id;
    }

    public static boolean isOceanicBiome(Biome b)
    {
        return OCEAN == b || DEEP_OCEAN == b;
    }

    public static boolean isMountainBiome(Biome b)
    {
        return MOUNTAINS == b || MOUNTAINS_EDGE == b;
    }

    public static boolean isBeachBiome(Biome b)
    {
        return BEACH == b || GRAVEL_BEACH == b;
    }

    public static List<Biome> getSpawnBiomes()
    {
        return SPAWN_BIOMES;
    }

    public static List<Biome> getWorldGenBiomes()
    {
        return WORLD_GEN_BIOMES;
    }

    private static void register(IForgeRegistry<Biome> r, Biome biome, boolean isSpawn, boolean isWorldGen, BiomeDictionary.Type... types)
    {
        r.register(biome.setRegistryName(MOD_ID, biome.field_76791_y.replace(MOD_NAME + " ", "").replace(' ', '_').toLowerCase()));

        // Other biome registration stuff
        BiomeDictionary.addTypes(biome, types);

        // These need to happen after the biomes are constructed, otherwise they will be null
        if (isSpawn)
        {
            SPAWN_BIOMES.add(biome);
        }
        if (isWorldGen)
        {
            WORLD_GEN_BIOMES.add(biome);
        }
    }

    private BiomesTFC() {}
}
