/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.world.classic.worldgen;

import java.util.Random;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.state.IBlockState;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.EnumSkyBlock;
import net.minecraft.world.World;
import net.minecraft.world.gen.feature.WorldGenerator;
import net.minecraftforge.common.BiomeDictionary;

import net.dries007.tfc.api.types.Plant;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.objects.blocks.plants.*;
import net.dries007.tfc.world.classic.ClimateTFC;
import net.dries007.tfc.world.classic.chunkdata.ChunkDataTFC;

@ParametersAreNonnullByDefault
public class WorldGenPlantTFC extends WorldGenerator
{
    private Plant plant;

    public void setGeneratedPlant(Plant plantIn)
    {
        this.plant = plantIn;
    }

    public boolean func_180709_b(World worldIn, Random rand, BlockPos position)
    {
        if (plant.getIsClayMarking()) return false;
        if (plant.getIsSwampPlant() && (/*!ClimateTFC.isSwamp(worldIn, position) ||*/ !BiomeDictionary.hasType(worldIn.func_180494_b(position), BiomeDictionary.Type.SWAMP)))
            return false;

        switch (plant.getPlantType())
        {
            case MUSHROOM:
            {
                BlockMushroomTFC plantBlock = BlockMushroomTFC.get(plant);
                IBlockState state = plantBlock.func_176223_P();

                for (int i = 0; i < ChunkDataTFC.getRainfall(worldIn, position) / 16; ++i)
                {
                    BlockPos blockpos = position.func_177982_a(rand.nextInt(4) - rand.nextInt(4), rand.nextInt(4) - rand.nextInt(4), rand.nextInt(4) - rand.nextInt(4));

                    if (plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                        plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos)) &&
                        worldIn.func_175623_d(blockpos) &&
                        plantBlock.func_176196_c(worldIn, blockpos))
                    {
                        int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                        func_175903_a(worldIn, blockpos, state.func_177226_a(BlockPlantTFC.AGE, plantAge));
                    }
                }
                break;
            }
            case SHORT_GRASS:
            {
                BlockShortGrassTFC plantBlock = BlockShortGrassTFC.get(plant);
                IBlockState state = plantBlock.func_176223_P();

                for (int i = 0; i < ChunkDataTFC.getRainfall(worldIn, position) / 4; ++i)
                {
                    BlockPos blockpos = position.func_177982_a(rand.nextInt(7) - rand.nextInt(7), rand.nextInt(4) - rand.nextInt(4), rand.nextInt(7) - rand.nextInt(7));

                    if (plant.isValidGrowthTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                        plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos)) &&
                        worldIn.func_175623_d(blockpos) &&
                        plantBlock.func_180671_f(worldIn, blockpos, state))
                    {
                        int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                        func_175903_a(worldIn, blockpos, state.func_177226_a(BlockShortGrassTFC.AGE, plantAge));
                    }
                }
                break;
            }
            case TALL_GRASS:
            {
                BlockTallGrassTFC plantBlock = BlockTallGrassTFC.get(plant);
                IBlockState state = plantBlock.func_176223_P();

                for (int i = 0; i < ChunkDataTFC.getRainfall(worldIn, position) / 16; ++i)
                {
                    BlockPos blockpos = position.func_177982_a(rand.nextInt(7) - rand.nextInt(7), rand.nextInt(4) - rand.nextInt(4), rand.nextInt(7) - rand.nextInt(7));

                    int j = 1 + rand.nextInt(plant.getMaxHeight());

                    for (int k = 0; k < j; ++k)
                    {
                        if (plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                            plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos.func_177981_b(k))) &&
                            worldIn.func_175623_d(blockpos.func_177981_b(k)) &&
                            plantBlock.func_180671_f(worldIn, blockpos.func_177981_b(k), state))
                        {
                            int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                            func_175903_a(worldIn, blockpos.func_177981_b(k), state.func_177226_a(BlockShortGrassTFC.AGE, plantAge));
                        }
                    }
                }
                break;
            }
            case CREEPING:
            {
                BlockCreepingPlantTFC plantBlock = BlockCreepingPlantTFC.get(plant);
                IBlockState state = plantBlock.func_176223_P();

                for (int i = 0; i < ChunkDataTFC.getRainfall(worldIn, position) / 16; ++i)
                {
                    BlockPos blockpos = position.func_177982_a(rand.nextInt(4) - rand.nextInt(4), rand.nextInt(4) - rand.nextInt(4), rand.nextInt(4) - rand.nextInt(4));

                    if (plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                        plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos)) &&
                        worldIn.func_175623_d(blockpos) &&
                        plantBlock.func_180671_f(worldIn, blockpos, state) &&
                        !BlocksTFC.isSand(worldIn.func_180495_p(blockpos.func_177977_b())))
                    {
                        int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                        func_175903_a(worldIn, blockpos, state.func_177226_a(BlockCreepingPlantTFC.AGE, plantAge));
                    }
                }
                break;
            }
            case HANGING:
            {
                BlockHangingPlantTFC plantBlock = BlockHangingPlantTFC.get(plant);
                IBlockState state = plantBlock.func_176223_P();

                for (int i = 0; i < ChunkDataTFC.getRainfall(worldIn, position) / 4; ++i)
                {
                    BlockPos blockpos = position.func_177982_a(rand.nextInt(7) - rand.nextInt(7), rand.nextInt(16), rand.nextInt(7) - rand.nextInt(7));

                    if (plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                        plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos)) &&
                        worldIn.func_175623_d(blockpos) &&
                        plantBlock.func_180671_f(worldIn, blockpos, state))
                    {
                        int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                        func_175903_a(worldIn, blockpos, state.func_177226_a(BlockHangingPlantTFC.AGE, plantAge));
                    }
                }
                break;
            }
            case REED:
            case REED_SEA:
            {
                BlockPlantTFC plantBlock = BlockPlantTFC.get(plant);
                IBlockState state = plantBlock.func_176223_P();

                for (int i = 0; i < ChunkDataTFC.getRainfall(worldIn, position) / 16; ++i)
                {
                    BlockPos blockpos = position.func_177982_a(rand.nextInt(7) - rand.nextInt(7), rand.nextInt(4) - rand.nextInt(4), rand.nextInt(7) - rand.nextInt(7));

                    if (plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                        plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos)) &&
                        worldIn.func_175623_d(blockpos) &&
                        worldIn.func_180495_p(blockpos.func_177977_b()).func_177230_c().canSustainPlant(state, worldIn, blockpos.func_177977_b(), EnumFacing.UP, plantBlock))
                    {
                        int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                        func_175903_a(worldIn, blockpos, state.func_177226_a(BlockPlantTFC.AGE, plantAge));
                    }
                }
                break;
            }
            case TALL_REED:
            case TALL_REED_SEA:
            {
                BlockTallPlantTFC plantBlock = BlockTallPlantTFC.get(plant);
                IBlockState state = plantBlock.func_176223_P();

                for (int i = 0; i < ChunkDataTFC.getRainfall(worldIn, position) / 16; ++i)
                {
                    BlockPos blockpos = position.func_177982_a(rand.nextInt(7) - rand.nextInt(7), rand.nextInt(4) - rand.nextInt(4), rand.nextInt(7) - rand.nextInt(7));

                    int j = 1 + rand.nextInt(plant.getMaxHeight());

                    for (int k = 0; k < j; ++k)
                    {
                        if (plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                            plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos.func_177981_b(k))) &&
                            worldIn.func_175623_d(blockpos.func_177981_b(k)) &&
                            plantBlock.func_180671_f(worldIn, blockpos.func_177981_b(k), state))
                        {
                            int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                            func_175903_a(worldIn, blockpos.func_177981_b(k), state.func_177226_a(BlockTallPlantTFC.AGE, plantAge));
                        }
                    }
                }
                break;
            }
            case DESERT:
            {
                BlockPlantTFC plantBlock = BlockPlantTFC.get(plant);
                IBlockState state = plantBlock.func_176223_P();

                for (int i = 0; i < ChunkDataTFC.getRainfall(worldIn, position); ++i)
                {
                    BlockPos blockpos = position.func_177982_a(rand.nextInt(7) - rand.nextInt(7), rand.nextInt(4) - rand.nextInt(4), rand.nextInt(7) - rand.nextInt(7));

                    if (plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                        plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos)) &&
                        worldIn.func_175623_d(blockpos) &&
                        !BiomeDictionary.hasType(worldIn.func_180494_b(blockpos), BiomeDictionary.Type.BEACH) &&
                        plantBlock.func_180671_f(worldIn, blockpos, state))
                    {
                        int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                        func_175903_a(worldIn, blockpos, state.func_177226_a(BlockPlantTFC.AGE, plantAge));
                    }
                }
                break;
            }
            case DESERT_TALL_PLANT:
            {
                BlockTallPlantTFC plantBlock = BlockTallPlantTFC.get(plant);
                IBlockState state = plantBlock.func_176223_P();

                for (int i = 0; i < ChunkDataTFC.getRainfall(worldIn, position); ++i)
                {
                    BlockPos blockpos = position.func_177982_a(rand.nextInt(7) - rand.nextInt(7), rand.nextInt(4) - rand.nextInt(4), rand.nextInt(7) - rand.nextInt(7));

                    int j = 1 + rand.nextInt(plant.getMaxHeight());

                    for (int k = 0; k < j; ++k)
                    {
                        if (plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                            plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos.func_177981_b(k))) &&
                            worldIn.func_175623_d(blockpos.func_177981_b(k)) &&
                            plantBlock.func_180671_f(worldIn, blockpos.func_177981_b(k), state))
                        {
                            int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                            func_175903_a(worldIn, blockpos.func_177981_b(k), state.func_177226_a(BlockTallPlantTFC.AGE, plantAge));
                        }
                    }
                }
                break;
            }
            case DRY:
            {
                BlockPlantTFC plantBlock = BlockPlantTFC.get(plant);
                IBlockState state = plantBlock.func_176223_P();

                for (int i = 0; i < ChunkDataTFC.getRainfall(worldIn, position); ++i)
                {
                    BlockPos blockpos = position.func_177982_a(rand.nextInt(7) - rand.nextInt(7), rand.nextInt(4) - rand.nextInt(4), rand.nextInt(7) - rand.nextInt(7));

                    if (plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                        plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos)) &&
                        worldIn.func_175623_d(blockpos) &&
                        !BiomeDictionary.hasType(worldIn.func_180494_b(blockpos), BiomeDictionary.Type.BEACH) &&
                        plantBlock.func_180671_f(worldIn, blockpos, state))
                    {
                        int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                        func_175903_a(worldIn, blockpos, state.func_177226_a(BlockPlantTFC.AGE, plantAge));
                    }
                }
                break;
            }
            case DRY_TALL_PLANT:
            {
                BlockTallPlantTFC plantBlock = BlockTallPlantTFC.get(plant);
                IBlockState state = plantBlock.func_176223_P();

                for (int i = 0; i < ChunkDataTFC.getRainfall(worldIn, position); ++i)
                {
                    BlockPos blockpos = position.func_177982_a(rand.nextInt(7) - rand.nextInt(7), rand.nextInt(4) - rand.nextInt(4), rand.nextInt(7) - rand.nextInt(7));

                    int j = 1 + rand.nextInt(plant.getMaxHeight());

                    for (int k = 0; k < j; ++k)
                    {
                        if (plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                            plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos.func_177981_b(k))) &&
                            worldIn.func_175623_d(blockpos.func_177981_b(k)) &&
                            plantBlock.func_180671_f(worldIn, blockpos.func_177981_b(k), state))
                        {
                            int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                            func_175903_a(worldIn, blockpos.func_177981_b(k), state.func_177226_a(BlockTallPlantTFC.AGE, plantAge));
                        }
                    }
                }
                break;
            }
            case TALL_PLANT:
            {
                BlockTallPlantTFC plantBlock = BlockTallPlantTFC.get(plant);
                IBlockState state = plantBlock.func_176223_P();

                for (int i = 0; i < ChunkDataTFC.getRainfall(worldIn, position) / 16; ++i)
                {
                    BlockPos blockpos = position.func_177982_a(rand.nextInt(7) - rand.nextInt(7), rand.nextInt(4) - rand.nextInt(4), rand.nextInt(7) - rand.nextInt(7));

                    int j = 1 + rand.nextInt(plant.getMaxHeight());

                    for (int k = 0; k < j; ++k)
                    {
                        if (plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                            plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos.func_177981_b(k))) &&
                            worldIn.func_175623_d(blockpos.func_177981_b(k)) &&
                            plantBlock.func_180671_f(worldIn, blockpos.func_177981_b(k), state))
                        {
                            int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                            func_175903_a(worldIn, blockpos.func_177981_b(k), state.func_177226_a(BlockTallPlantTFC.AGE, plantAge));
                        }
                    }
                }
                break;
            }
            case WATER:
            case WATER_SEA:
            {
                BlockWaterPlantTFC plantBlock = BlockWaterPlantTFC.get(plant);
                IBlockState state = plantBlock.func_176223_P();
                IBlockState water = plant.getWaterType();

                int depth = plant.getValidWaterDepth(worldIn, position, water);
                if (depth == -1) return false;

                BlockPos blockpos = position.func_177982_a(0, -depth + 1, 0);

                if (plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                    plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos)) &&
                    plantBlock.func_176196_c(worldIn, blockpos))
                {
                    int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                    func_175903_a(worldIn, blockpos, state.func_177226_a(BlockWaterPlantTFC.AGE, plantAge));
                }
                break;
            }
            case EMERGENT_TALL_WATER:
            case EMERGENT_TALL_WATER_SEA:
            {
                BlockEmergentTallWaterPlantTFC plantBlock = BlockEmergentTallWaterPlantTFC.get(plant);
                IBlockState state = plantBlock.func_176223_P();
                IBlockState water = plant.getWaterType();

                int depth = plant.getValidWaterDepth(worldIn, position, water);
                if (depth == -1) return false;
                BlockPos blockpos = position.func_177982_a(0, -depth + 1, 0);

                if (plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                    plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos)) &&
                    plantBlock.func_176196_c(worldIn, blockpos))
                {
                    int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                    func_175903_a(worldIn, blockpos, state.func_177226_a(BlockEmergentTallWaterPlantTFC.AGE, plantAge));
                    if (rand.nextInt(3) < plantAge && plantBlock.func_176473_a(worldIn, blockpos, state, worldIn.field_72995_K))
                        func_175903_a(worldIn, blockpos.func_177984_a(), state);
                }
                break;
            }
            case TALL_WATER:
            case TALL_WATER_SEA:
            {
                BlockTallWaterPlantTFC plantBlock = BlockTallWaterPlantTFC.get(plant);
                IBlockState state = plantBlock.func_176223_P();
                IBlockState water = plant.getWaterType();

                int depth = plant.getValidWaterDepth(worldIn, position, water);
                if (depth == -1) return false;
                BlockPos blockpos = position.func_177982_a(0, -depth + 1, 0);

                if (plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                    plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos)) &&
                    plantBlock.func_176196_c(worldIn, blockpos))
                {
                    int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                    func_175903_a(worldIn, blockpos, state.func_177226_a(BlockTallPlantTFC.AGE, plantAge));
                    if (rand.nextInt(4) < plantAge && plantBlock.func_176473_a(worldIn, blockpos, state, worldIn.field_72995_K))
                        func_175903_a(worldIn, blockpos.func_177984_a(), state);
                }
                break;
            }
            case FLOATING:
            {
                BlockFloatingWaterTFC plantBlock = BlockFloatingWaterTFC.get(plant);
                IBlockState state = plantBlock.func_176223_P();
                IBlockState water = plant.getWaterType();

                for (int i = 0; i < 8; ++i)
                {
                    final BlockPos blockpos = position.func_177982_a(rand.nextInt(7) - rand.nextInt(7), 0, rand.nextInt(7) - rand.nextInt(7));

                    if (plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                        plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos)) &&
                        worldIn.func_175623_d(blockpos) &&
                        plantBlock.func_176196_c(worldIn, blockpos) &&
                        plant.isValidFloatingWaterDepth(worldIn, blockpos, water))
                    {
                        int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                        func_175903_a(worldIn, blockpos, state.func_177226_a(BlockFloatingWaterTFC.AGE, plantAge));
                    }
                }
                break;
            }
            case FLOATING_SEA:
            {
                BlockFloatingWaterTFC plantBlock = BlockFloatingWaterTFC.get(plant);
                IBlockState state = plantBlock.func_176223_P();
                IBlockState water = plant.getWaterType();

                for (int i = 0; i < 128; ++i)
                {
                    final BlockPos blockpos = position.func_177982_a(rand.nextInt(7) - rand.nextInt(7), 0, rand.nextInt(7) - rand.nextInt(7));

                    if (plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                        plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos)) &&
                        worldIn.func_175623_d(blockpos) &&
                        plantBlock.func_176196_c(worldIn, blockpos) &&
                        plant.isValidFloatingWaterDepth(worldIn, blockpos, water))
                    {
                        int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                        func_175903_a(worldIn, blockpos, state.func_177226_a(BlockFloatingWaterTFC.AGE, plantAge));
                    }
                }
                break;
            }
            case CACTUS:
            {
                BlockCactusTFC plantBlock = BlockCactusTFC.get(plant);
                IBlockState state = plantBlock.func_176223_P();

                for (int i = 0; i < ChunkDataTFC.getRainfall(worldIn, position) / 8; ++i)
                {
                    BlockPos blockpos = position.func_177982_a(rand.nextInt(7) - rand.nextInt(7), rand.nextInt(4) - rand.nextInt(4), rand.nextInt(7) - rand.nextInt(7));

                    int j = 1 + rand.nextInt(plant.getMaxHeight());

                    for (int k = 0; k < j; ++k)
                    {
                        if (plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                            plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos.func_177981_b(k))) &&
                            worldIn.func_175623_d(blockpos.func_177981_b(k)) &&
                            plantBlock.func_180671_f(worldIn, blockpos.func_177981_b(k), state))
                        {
                            int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                            func_175903_a(worldIn, blockpos.func_177981_b(k), state.func_177226_a(BlockCactusTFC.AGE, plantAge));
                        }
                    }
                }
                break;
            }
            case EPIPHYTE:
            {
                BlockEpiphyteTFC plantBlock = BlockEpiphyteTFC.get(plant);

                for (int i = 0; i < ChunkDataTFC.getRainfall(worldIn, position) / 4; ++i)
                {
                    BlockPos blockpos = position.func_177982_a(rand.nextInt(7) - rand.nextInt(7), rand.nextInt(16), rand.nextInt(7) - rand.nextInt(7));

                    if (plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                        plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos)) &&
                        worldIn.func_180495_p(blockpos).func_177230_c().func_176200_f(worldIn, blockpos) &&
                        plantBlock.func_176196_c(worldIn, blockpos))
                    {
                        int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                        func_175903_a(worldIn, blockpos, plantBlock.getStateForWorldGen(worldIn, blockpos).func_177226_a(BlockEpiphyteTFC.AGE, plantAge));
                    }
                }
                break;
            }
            default:
            {
                BlockPlantTFC plantBlock = BlockPlantTFC.get(plant);
                IBlockState state = plantBlock.func_176223_P();

                for (int i = 0; i < ChunkDataTFC.getRainfall(worldIn, position) / 16; ++i)
                {
                    BlockPos blockpos = position.func_177982_a(rand.nextInt(7) - rand.nextInt(7), rand.nextInt(4) - rand.nextInt(4), rand.nextInt(7) - rand.nextInt(7));

                    if (plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos)) &&
                        plant.isValidSunlight(worldIn.func_175642_b(EnumSkyBlock.SKY, blockpos)) &&
                        worldIn.func_175623_d(blockpos) &&
                        plantBlock.func_180671_f(worldIn, blockpos, state))
                    {
                        int plantAge = plant.getAgeForWorldgen(rand, ClimateTFC.getHeightAdjustedTemp(worldIn, blockpos));
                        func_175903_a(worldIn, blockpos, state.func_177226_a(BlockPlantTFC.AGE, plantAge));
                    }
                }
            }
        }
        return true;
    }
}
